/*
  File Name     : ft602_uvc_top.v 
  Department    : IC Design, FTDI SGP
  Author        : Do Ngoc Duong
  History       : 25 Aug. 2016 - Initial Version
                  13 Dec. 2016 - 4 channels pattern  

  Description   : This is the top of ft602 UVC FPGA reference code 
*/
//===========================================================
module ft602_uvc_top(
  // CLOCK  //////////
  input         CLK50M,
  // LED & Debug        //////////
  output [7:0]  LEDG,
  output [5:0]  DEBUG,
  output [3:0]  FRDROP,
  input  [7:0]  DIPSW,
  input  [2:0]  PBUTT,
  // KEY                //////////
  input         HRST_N, // On board reset  
  inout         SRST_N, // From UMFT602 board
  // I2C                //////////
  inout         I2C_SCL,
  inout         I2C_SDA,
  inout         IRQ,
/////////// FT602       //////////
  input         CLK,
  inout [31:0]  DATA,
  output [3:0]  BE,
  input         RXF_N,  // ACK_N
  input         TXE_N,
  output        WR_N,   // REQ_N
  output        SIWU_N,
  output        RD_N,
  output        OE_N
);
//===Parameter
localparam MEM_ADB = 12;
localparam MEM_LEN = 4096; 
//===Clock & Reset
wire clk50;
wire clk100;
wire fhd_clk;
wire hd_clk;
wire sd_clk;
wire reset;
wire plllocked;
wire inclock;
assign inclock = CLK50M; 
//
`ifdef ALTERA_FPGA 
assign reset = !HRST_N;
`else 
assign reset = HRST_N;
`endif
assign SRST_N = (!plllocked) ? 1'b0 : 1'bz;
//
//=== PLL 
sys_pll  i0_sys_pll (
  .rst (reset),
  .refclk (inclock),
  .outclk_0 (clk50),
  .outclk_1 (clk100),
  .outclk_2 (fhd_clk),
  .outclk_3 (hd_clk),
  .outclk_4 (sd_clk),
  .locked(plllocked)
);
//=== I2C
wire  tc_rst_n;
wire  rst_n;
assign rst_n = tc_rst_n & plllocked; 
wire  wakeup;
wire  irst_n;
assign irst_n = rst_n & wakeup;
//
wire  reg_wr;
wire  reg_rd;
wire  [7:0]   reg_adr;
wire  [127:0] reg_di;
wire          reg_rdy;
wire  [127:0] reg_do;
//
wire tp_i2c_oe;
wire tp_i2c_sdo;
wire tc_i2c_scl;
wire tc_i2c_sdi;
wire tc_i2c_irq; 

ft602_i2c_slv i1_ft602_i2c_slv (
  //i2c interface
  .clk    (clk50),
  .rst_n  (rst_n),
  // 
  .reg_wr (reg_wr),
  .reg_rd (reg_rd),
  .reg_adr(reg_adr),
  .reg_di (reg_di),
  .reg_rdy(reg_rdy),
  .reg_do (reg_do),
  //Registers interface
  .i2c_scl(tc_i2c_scl),
  .i2c_sdi(tc_i2c_sdi),
  .i2c_sdo(tp_i2c_sdo),
  .i2c_oe (tp_i2c_oe)
);

//=== IO 
wire [31:0] tp_data;
wire [3:0]  tp_be;
wire tp_dt_oe_n;
wire tp_be_oe_n;
wire tp_siwu_n;
wire tp_wr_n;
wire tp_rd_n;
wire tp_oe_n;
wire tp_int_ena;
wire tp_int_irq;
wire [31:0] tc_data;
wire [3:0]  tc_be;
wire tc_clk;
wire tc_txe_n;
wire tc_rxf_n;
wire tc_mltcn;
//
wire mltcn; 
//
ft602_uvc_io i2_ft602_uvc_io (
  //GPIO Control Signals
  `ifdef ALTERA_FPGA   
  .HRST_N     (HRST_N),
  `else 
  .HRST_N     (!HRST_N),
  `endif
  .SRST_N     (1'b1), 
  .MLTCN      (mltcn),
  // FIFO Slave interface 
  .CLK        (CLK),
  .DATA       (DATA),
  .BE         (BE),
  .RXF_N      (RXF_N),
  .TXE_N      (TXE_N),
  .WR_N       (WR_N),
  .SIWU_N     (SIWU_N),
  .RD_N       (RD_N),
  .OE_N       (OE_N),
  .I2C_SCL    (I2C_SCL),
  .I2C_SDA    (I2C_SDA),
  // 
  .IRQ        (),//(IRQ),
  // From chip internal 
  .tp_data    (tp_data),
  .tp_be      (tp_be),
  .tp_dt_oe_n (tp_dt_oe_n),
  .tp_be_oe_n (tp_be_oe_n),
  .tp_siwu_n  (tp_siwu_n),
  .tp_wr_n    (tp_wr_n),
  .tp_rd_n    (tp_rd_n),
  .tp_oe_n    (tp_oe_n),
  //
  .tp_i2c_oe  (tp_i2c_oe),
  .tp_i2c_sdo (tp_i2c_sdo),
  //To chip internal 
  .tc_i2c_scl (tc_i2c_scl),
  .tc_i2c_sdi (tc_i2c_sdi),
  //
  .tp_int_ena (tp_int_ena),
  .tp_int_irq (tp_int_irq), 
  // 
  .tc_data    (tc_data),
  .tc_be      (tc_be),
  .tc_rst_n   (tc_rst_n),
  .tc_clk     (tc_clk),
  .tc_txe_n   (tc_txe_n),
  .tc_rxf_n   (tc_rxf_n),
  //
  .tc_mltcn   (tc_mltcn)
);
//FIFO Protocol control 
// 
wire ch0_req;
wire ch1_req;
wire ch2_req;
wire ch3_req;
wire ch0_nempt;
wire ch1_nempt;
wire ch2_nempt;
wire ch3_nempt;
wire [35:0] ch0_dat;
wire [35:0] ch1_dat;
wire [35:0] ch2_dat;
wire [35:0] ch3_dat;
// 
wire ififord;
wire ififowr;
wire [1:0]  ififowrid;
wire [3:0]  ififoafull = 4'h0;
wire [3:0]  ififonempt;
wire [35:0] ififo_wdat;
wire [35:0] ififo_rdat;
//
wire prefena;
wire prefreq;
wire prefmod;
wire[ 1:0]  prefchn;
wire[ 3:0]  prefnempt;
wire[35:0]  prefdout;
//
wire [3:0]  tc_mst_rd_n = 4'hF;
wire [3:0]  tc_mst_wr_n; 
//
wire [3:0] tp_debug_sig;
//
ft602_uvc_fsm i3_ft602_uvc_fsm (
  // IO interface 
  .rst_n      (irst_n),
  .clk        (tc_clk),
  .txe_n      (tc_txe_n),
  .rxf_n      (tc_rxf_n),
  .idata      (tc_data),
  .ibe        (tc_be),
  //
  .mltcn      (tc_mltcn),
  .stren      (1'b0),
  .r_oob      (1'b0), //(tc_r_oob),
  .w_oob      (1'b0), //(tc_w_oob),
  .mst_rd_n   (tc_mst_rd_n),
  .mst_wr_n   (tc_mst_wr_n),
  // 
  .odata      (tp_data),
  .obe        (tp_be),
  .dt_oe_n    (tp_dt_oe_n),
  .be_oe_n    (tp_be_oe_n),
  .siwu_n     (tp_siwu_n),
  .wr_n       (tp_wr_n),
  .rd_n       (tp_rd_n),
  .oe_n       (tp_oe_n),
  // 
  .tp_debug_sig(tp_debug_sig),
  .ch0_vld    (),
  .ch1_vld    (),
  .ch2_vld    (),
  .ch3_vld    (),
  .chk_data   (),
  .chk_err    (4'h0),    
  // Check Data interface 
  // internal FIFO control interface 
  .ififoafull (ififoafull),
  .ififonempt (ififonempt),
  .ififowr    (ififowr),
  .ififowrid  (ififowrid),
  .ififo_wdat (ififo_wdat),
  //
  .prefena   (prefena),
  .prefreq   (prefreq),
  .prefmod   (prefmod),
  .prefchn   (prefchn),
  .prefnempt (prefnempt),
  .prefdout  (prefdout)
);
//
ft602_pre_fet i4_ft602_pre_fet (
  .clk      (tc_clk),
  .rst_n    (irst_n),
   //Flow control interface
  .prefena  (prefena),
  .prefreq  (prefreq),
  .prefchn  (prefchn),
  .prefnempt(prefnempt),
  .prefdout (prefdout),
   //Streaming generate interface 
  .gen0req  (ch0_req),
  .gen1req  (ch1_req),
  .gen2req  (ch2_req),
  .gen3req  (ch3_req),
  .chn0nempt(ch0_nempt),
  .chn1nempt(ch1_nempt),
  .chn2nempt(ch2_nempt),
  .chn3nempt(ch3_nempt),
  .gen0dat  (ch0_dat),
  .gen1dat  (ch1_dat),
  .gen2dat  (ch2_dat),
  .gen3dat  (ch3_dat)
);
//
wire [3:0] alive;
wire [7:0] mode_select; 
wire [15:0] cn0_control;
wire [31:0] cn0_v_width;
wire [15:0] cn0_l_valid;
wire [15:0] cn0_l_sg_eg; 
wire [15:0] cn0_h_blank;
wire [15:0] cn0_d_valid;
wire [15:0] cn0_d_sg_eg; 
//
wire [15:0] ch0_high_mark;
wire [15:0] ch0_low_mark;
wire        ch0_frame_drop;
//
ft602_pch_inp i5_ch0(
  //
  .rst_n    (irst_n),
  .sys_clk  (clk50),
  .fhd_clk  (fhd_clk),
  .hd_clk   (hd_clk),
  .sd_clk   (sd_clk),
  //Registers 
  .mode_select  (mode_select),  
  .control  (cn0_control),
  .v_width  (cn0_v_width),
  .l_valid  (cn0_l_valid),
  .l_sg_eg  (cn0_l_sg_eg),
  .h_blank  (cn0_h_blank),
  .d_valid  (cn0_d_valid),
  .d_sg_eg  (cn0_d_sg_eg), 
  //
  .high_mark(ch0_high_mark),
  .low_mark (ch0_low_mark),
  .alive    (alive[0]),
  .frame_drop(ch0_frame_drop),
  //
  // HDMI card interface  
  .hdmi_pclk  (1'b0),
  .hdmi_vsync (1'b0),
  .hdmi_hsync (1'b0),
  .hdmi_de    (1'b0),
  .hdmi_rd    (12'd0),
  .hdmi_gd    (12'd0),
  .hdmi_bd    (12'd0),
  //
  .fifo_rclk  (tc_clk),
  .fiford     (ch0_req),
  .fifonempt  (ch0_nempt),
  .fifo_rdata (ch0_dat) 
);
//
wire [15:0] cn1_control;
wire [31:0] cn1_v_width;
wire [15:0] cn1_l_valid;
wire [15:0] cn1_l_sg_eg; 
wire [15:0] cn1_h_blank;
wire [15:0] cn1_d_valid;
wire [15:0] cn1_d_sg_eg; 
//
wire [15:0] ch1_high_mark;
wire [15:0] ch1_low_mark;
wire        ch1_frame_drop;
//
ft602_pch_inp i6_ch1(
  //
  .rst_n    (irst_n),
  .sys_clk  (clk50),
  .fhd_clk  (fhd_clk),
  .hd_clk   (hd_clk),
  .sd_clk   (sd_clk),
  //Registers 
  .mode_select  (mode_select),  
  .control  (cn1_control),
  .v_width  (cn1_v_width),
  .l_valid  (cn1_l_valid),
  .l_sg_eg  (cn1_l_sg_eg),
  .h_blank  (cn1_h_blank),
  .d_valid  (cn1_d_valid),
  .d_sg_eg  (cn1_d_sg_eg),
  //
  .high_mark(ch1_high_mark),
  .low_mark (ch1_low_mark),
  .alive    (alive[1]),  
  .frame_drop(ch1_frame_drop),
  //
  // HDMI card interface  
  .hdmi_pclk  (1'b0),
  .hdmi_vsync (1'b0),
  .hdmi_hsync (1'b0),
  .hdmi_de    (1'b0),
  .hdmi_rd    (12'd0),
  .hdmi_gd    (12'd0),
  .hdmi_bd    (12'd0),
  //
  .fifo_rclk  (tc_clk),
  .fiford     (ch1_req),
  .fifonempt  (ch1_nempt),
  .fifo_rdata (ch1_dat) 
);
//
wire [15:0] cn2_control;
wire [31:0] cn2_v_width;
wire [15:0] cn2_l_valid;
wire [15:0] cn2_l_sg_eg; 
wire [15:0] cn2_h_blank;
wire [15:0] cn2_d_valid;
wire [15:0] cn2_d_sg_eg; 
//
wire [15:0] ch2_high_mark;
wire [15:0] ch2_low_mark;
wire        ch2_frame_drop;
//
ft602_pch_inp i7_ch2(
  //
  .rst_n    (irst_n),
  .sys_clk  (clk50),
  .fhd_clk  (fhd_clk),
  .hd_clk   (hd_clk),
  .sd_clk   (sd_clk),
  //Registers 
  .mode_select  (mode_select),  
  .control  (cn2_control),
  .v_width  (cn2_v_width),
  .l_valid  (cn2_l_valid),
  .l_sg_eg  (cn2_l_sg_eg),
  .h_blank  (cn2_h_blank),
  .d_valid  (cn2_d_valid),
  .d_sg_eg  (cn2_d_sg_eg),
  //
  .high_mark(ch2_high_mark),
  .low_mark (ch2_low_mark),
  .alive    (alive[2]),  
  .frame_drop(ch2_frame_drop),
  // HDMI card interface  
  .hdmi_pclk  (1'b0),
  .hdmi_vsync (1'b0),
  .hdmi_hsync (1'b0),
  .hdmi_de    (1'b0),
  .hdmi_rd    (12'd0),
  .hdmi_gd    (12'd0),
  .hdmi_bd    (12'd0),
  //
  .fifo_rclk  (tc_clk),
  .fiford     (ch2_req),
  .fifonempt  (ch2_nempt),
  .fifo_rdata (ch2_dat) 
);
//
wire [15:0] cn3_control;
wire [31:0] cn3_v_width;
wire [15:0] cn3_l_valid;
wire [15:0] cn3_l_sg_eg; 
wire [15:0] cn3_h_blank;
wire [15:0] cn3_d_valid;
wire [15:0] cn3_d_sg_eg; 
//
wire [15:0] ch3_high_mark;
wire [15:0] ch3_low_mark;
wire        ch3_frame_drop;
//
ft602_pch_inp i8_ch3(
  //
  .rst_n    (irst_n),
  .sys_clk  (clk50),
  .fhd_clk  (fhd_clk),
  .hd_clk   (hd_clk),
  .sd_clk   (sd_clk),
  //Registers 
  .mode_select  (mode_select),  
  .control  (cn3_control),
  .v_width  (cn3_v_width),
  .l_valid  (cn3_l_valid),
  .l_sg_eg  (cn3_l_sg_eg),
  .h_blank  (cn3_h_blank),
  .d_valid  (cn3_d_valid),
  .d_sg_eg  (cn3_d_sg_eg),
  //
  .high_mark(ch3_high_mark),
  .low_mark (ch3_low_mark),
  .alive    (alive[3]),
  .frame_drop(ch3_frame_drop),
  //
  // HDMI card interface  
  .hdmi_pclk  (1'b0),
  .hdmi_vsync (1'b0),
  .hdmi_hsync (1'b0),
  .hdmi_de    (1'b0),
  .hdmi_rd    (12'd0),
  .hdmi_gd    (12'd0),
  .hdmi_bd    (12'd0),
  //
  .fifo_rclk  (tc_clk),
  .fiford     (ch3_req),
  .fifonempt  (ch3_nempt),
  .fifo_rdata (ch3_dat) 
);
//
wire [7:0] power_stat;
assign tc_mst_wr_n = 4'h0;
assign ififonempt  = {ch3_nempt,ch2_nempt,ch1_nempt,ch0_nempt};
//Register 
ft602_uvc_reg i9_ft602_uvc_reg (
  //i2c interface
  .clk        (clk50),
  .rst_n      (rst_n),
  .reg_wr     (reg_wr),
  .reg_rd     (reg_rd),
  .reg_adr    (reg_adr),
  .reg_di     (reg_di),
  .reg_rdy    (reg_rdy),
  .reg_do     (reg_do),
  //Registers interface
  .wakeup      (wakeup),
  .mode_select (mode_select),
  .cn0_control (cn0_control),
  .cn0_v_width (cn0_v_width),
  .cn0_l_valid (cn0_l_valid),
  .cn0_lsg_leg (cn0_l_sg_eg),
  .cn0_h_blank (cn0_h_blank),
  .cn0_d_valid (cn0_d_valid),
  .cn0_dsg_deg (cn0_d_sg_eg),
  //
  .cn1_control (cn1_control),
  .cn1_v_width (cn1_v_width),
  .cn1_l_valid (cn1_l_valid),
  .cn1_lsg_leg (cn1_l_sg_eg),
  .cn1_h_blank (cn1_h_blank),
  .cn1_d_valid (cn1_d_valid),
  .cn1_dsg_deg (cn1_d_sg_eg),
  //
  .cn2_control (cn2_control),
  .cn2_v_width (cn2_v_width),
  .cn2_l_valid (cn2_l_valid),
  .cn2_lsg_leg (cn2_l_sg_eg),
  .cn2_h_blank (cn2_h_blank),
  .cn2_d_valid (cn2_d_valid),
  .cn2_dsg_deg (cn2_d_sg_eg),
  //
  .cn3_control (cn3_control),
  .cn3_v_width (cn3_v_width),
  .cn3_l_valid (cn3_l_valid),
  .cn3_lsg_leg (cn3_l_sg_eg),
  .cn3_h_blank (cn3_h_blank),
  .cn3_d_valid (cn3_d_valid),
  .cn3_dsg_deg (cn3_d_sg_eg),
  //
  .ch0_low_mark  (ch0_low_mark),
  .ch0_high_mark (ch0_high_mark),
  .ch1_low_mark  (ch1_low_mark),
  .ch1_high_mark (ch1_high_mark),
  .ch2_low_mark  (ch2_low_mark),
  .ch2_high_mark (ch2_high_mark),
  .ch3_low_mark  (ch3_low_mark),
  .ch3_high_mark (ch3_high_mark),
  //
  .ch0_drop_trig (ch0_frame_drop),
  .ch1_drop_trig (ch1_frame_drop),
  .ch2_drop_trig (ch2_frame_drop),
  .ch3_drop_trig (ch3_frame_drop),  
  .frame_drop    (FRDROP),
  .power_stat    (power_stat),
  // 
  .DIPSW         (DIPSW),
  .PBUTT         (PBUTT),
  .int_irq       (tp_int_irq),
  .int_ena       (tp_int_ena) 
); 

//******************************************************
assign mltcn = mode_select[0];  
//*********************For Test only********************
assign LEDG[3:0] = power_stat[3:0];
assign LEDG[7:4] = alive;
assign DEBUG     = {tp_debug_sig,2'b00}; 
//
//******************************************************
endmodule 
